"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveModuleOptions = exports.createRawModuleRuleUses = void 0;
const node_assert_1 = __importDefault(require("node:assert"));
const node_path_1 = __importDefault(require("node:path"));
const devtool_1 = require("./devtool");
const toBuffer = (bufLike) => {
    if (Buffer.isBuffer(bufLike)) {
        return bufLike;
    }
    else if (typeof bufLike === "string") {
        return Buffer.from(bufLike);
    }
    throw new Error("Buffer or string expected");
};
function composeJsUse(uses, options) {
    if (!uses.length) {
        return null;
    }
    async function loader(data) {
        const payload = JSON.parse(data.toString("utf-8"));
        const loaderContextInternal = {
            source: payload.source,
            sourceMap: payload.sourceMap,
            resourcePath: payload.resourcePath,
            resourceQuery: payload.resourceQuery,
            resource: payload.resource,
            resourceFragment: payload.resourceFragment
        };
        let sourceBuffer = Buffer.from(loaderContextInternal.source);
        let meta = Buffer.from("");
        // Loader is executed from right to left
        for (const use of uses) {
            (0, node_assert_1.default)("loader" in use);
            /**
             * support loader as string
             */
            if (typeof use.loader === "string") {
                let loaderPath = require.resolve(use.loader, {
                    paths: [options.context]
                });
                use.loader = require(loaderPath);
            }
            const loaderContext = {
                ...loaderContextInternal,
                source: {
                    getCode() {
                        return sourceBuffer.toString("utf-8");
                    },
                    getBuffer() {
                        return sourceBuffer;
                    }
                },
                getOptions() {
                    return use.options;
                },
                useSourceMap: (0, devtool_1.isUseSourceMap)(options.devtool),
                rootContext: options.context,
                context: node_path_1.default.dirname(loaderContextInternal.resourcePath)
            };
            let loaderResult;
            if ((loaderResult = await Promise.resolve().then(() => use.loader.apply(loaderContext, [loaderContext])))) {
                const content = loaderResult.content;
                meta = meta.length > 0 ? meta : toBuffer(loaderResult.meta);
                sourceBuffer = toBuffer(content);
            }
        }
        const loaderResultPayload = {
            content: [...sourceBuffer],
            meta: [...meta]
        };
        return Buffer.from(JSON.stringify(loaderResultPayload), "utf-8");
    }
    loader.displayName = `NodeLoaderAdapter(${uses
        .map(item => {
        (0, node_assert_1.default)("loader" in item);
        return item.loader.displayName || item.loader.name || "unknown-loader";
    })
        .join(" -> ")})`;
    return {
        loader
    };
}
function createRawModuleRuleUses(uses, options) {
    return createRawModuleRuleUsesImpl([...uses].reverse(), options);
}
exports.createRawModuleRuleUses = createRawModuleRuleUses;
function createRawModuleRuleUsesImpl(uses, options) {
    if (!uses.length) {
        return [];
    }
    const index = uses.findIndex(use => "builtinLoader" in use);
    if (index < 0) {
        return [composeJsUse(uses, options)];
    }
    const before = uses.slice(0, index);
    const after = uses.slice(index + 1);
    return [
        composeJsUse(before, options),
        createNativeUse(uses[index]),
        ...createRawModuleRuleUsesImpl(after, options)
    ].filter((item) => Boolean(item));
}
function createNativeUse(use) {
    var _a;
    (0, node_assert_1.default)("builtinLoader" in use);
    if (use.builtinLoader === "sass-loader") {
        ((_a = use.options) !== null && _a !== void 0 ? _a : (use.options = {})).__exePath = require.resolve(`@tmp-sass-embedded/${process.platform}-${process.arch}/dart-sass-embedded/dart-sass-embedded${process.platform === "win32" ? ".bat" : ""}`);
    }
    return {
        builtinLoader: use.builtinLoader,
        options: JSON.stringify(use.options)
    };
}
function resolveModuleOptions(module = {}, options) {
    var _a;
    const rules = ((_a = module.rules) !== null && _a !== void 0 ? _a : []).map(rule => ({
        ...rule,
        uses: createRawModuleRuleUses(rule.uses || [], options)
    }));
    return {
        rules
    };
}
exports.resolveModuleOptions = resolveModuleOptions;
//# sourceMappingURL=module.js.map